@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 #
-#   Copyright (C) 2012 -2022  Christian Ledermann
+#   Copyright (C) 2012 -2024  Christian Ledermann
 #
 #   This library is free software; you can redistribute it and/or
 #   modify it under the terms of the GNU Lesser General Public
@@ -21,6 +20,8 @@
 import math
 import warnings
 from itertools import chain
+from typing import Any
+from typing import Hashable
 from typing import Iterable
 from typing import Iterator
 from typing import NoReturn
@@ -33,12 +34,14 @@ from typing import cast
 from pygeoif.exceptions import DimensionError
 from pygeoif.functions import centroid
 from pygeoif.functions import compare_coordinates
+from pygeoif.functions import compare_geo_interface
 from pygeoif.functions import convex_hull
 from pygeoif.functions import dedupe
 from pygeoif.functions import signed_area
 from pygeoif.types import Bounds
 from pygeoif.types import GeoCollectionInterface
 from pygeoif.types import GeoInterface
+from pygeoif.types import GeomType
 from pygeoif.types import GeoType
 from pygeoif.types import LineType
 from pygeoif.types import Point2D
@@ -49,6 +52,22 @@ from pygeoif.types import PolygonType
 class _Geometry:
     """Base Class for geometry objects."""
 
+    __slots__ = ("_geoms",)
+
+    _geoms: Hashable
+
+    def __setattr__(self, *args: Any) -> NoReturn:  # noqa: ANN401
+        msg = f"Attributes of {self.__class__.__name__} cannot be changed"
+        raise AttributeError(
+            msg,
+        )
+
+    def __delattr__(self, *args: Any) -> NoReturn:  # noqa: ANN401
+        msg = f"Attributes of {self.__class__.__name__} cannot be deleted"
+        raise AttributeError(
+            msg,
+        )
+
     def __str__(self) -> str:
         return self.wkt
 
@@ -67,8 +86,10 @@ class _Geometry:
                         "type",
                     ),
                     compare_coordinates(
-                        self.__geo_interface__["coordinates"],
-                        other.__geo_interface__.get("coordinates"),  # type: ignore
+                        coords=self.__geo_interface__["coordinates"],
+                        other=other.__geo_interface__.get(  # type: ignore [attr-defined]
+                            "coordinates",
+                        ),
                     ),
                 ),
             )
@@ -94,7 +115,7 @@ class _Geometry:
 
         Returns a representation of the smallest convex Polygon containing
         all the points in the object unless the number of points in the object
-        is less than three.
+        is fewer than three.
         For two points, the convex hull collapses to a LineString;
         for 1, to a Point.
         """
@@ -102,6 +123,7 @@ class _Geometry:
             warnings.warn(
                 "The convex Hull will only return the projection to"
                 " 2 dimensions xy coordinates",
+                stacklevel=2,
             )
 
         hull = convex_hull(self._prepare_hull())
@@ -109,7 +131,7 @@ class _Geometry:
             return None
         if len(hull) == 1:
             return Point(*hull[0])
-        return LineString(hull) if len(hull) == 2 else Polygon(hull)
+        return LineString(hull) if len(hull) == 2 else Polygon(hull)  # noqa: PLR2004
 
     @property
     def geom_type(self) -> str:
@@ -123,12 +145,14 @@ class _Geometry:
 
         Return None if the geometry is empty.
         """
-        raise NotImplementedError("Must be implemented by subclass")
+        msg = "Must be implemented by subclass"
+        raise NotImplementedError(msg)
 
     @property
     def is_empty(self) -> bool:
         """Return if this geometry is empty."""
-        raise NotImplementedError("Must be implemented by subclass")
+        msg = "Must be implemented by subclass"
+        raise NotImplementedError(msg)
 
     @property
     def wkt(self) -> str:
@@ -140,21 +164,23 @@ class _Geometry:
     @property
     def __geo_interface__(self) -> GeoInterface:
         if self.is_empty:
-            raise AttributeError("Empty Geometry")
+            msg = "Empty Geometry"
+            raise AttributeError(msg)
         return {
-            "type": self.geom_type,
+            "type": cast(GeomType, self.geom_type),
             "bbox": cast(Bounds, self.bounds),
             "coordinates": (),
         }
 
     @property
     def _wkt_coords(self) -> str:
-        raise NotImplementedError("Must be implemented by subclass")
+        msg = "Must be implemented by subclass"
+        raise NotImplementedError(msg)
 
     @property
     def _wkt_inset(self) -> str:
         """Return Z for 3 dimensional geometry or an empty string for 2 dimensions."""
-        return ""
+        return " Z " if self.has_z else " "
 
     @property
     def _wkt_type(self) -> str:
@@ -164,24 +190,28 @@ class _Geometry:
     @classmethod
     def _check_dict(cls, geo_interface: GeoInterface) -> None:
         if geo_interface["type"] != cls.__name__:
+            msg = f"You cannot assign {geo_interface['type']} to {cls.__name__}"
             raise ValueError(
-                f"You cannot assign {geo_interface['type']} to {cls.__name__}",
+                msg,
             )
 
     @classmethod
     def _from_dict(cls, geo_interface: GeoInterface) -> "_Geometry":
         cls._check_dict(geo_interface)
-        raise NotImplementedError("Must be implemented by subclass")
+        msg = "Must be implemented by subclass"
+        raise NotImplementedError(msg)
 
     @classmethod
     def _from_interface(cls, obj: GeoType) -> "_Geometry":
         return cls._from_dict(obj.__geo_interface__)
 
     def _prepare_hull(self) -> Iterable[Point2D]:
-        raise NotImplementedError("Must be implemented by subclass")
+        msg = "Must be implemented by subclass"
+        raise NotImplementedError(msg)
 
     def _get_bounds(self) -> Bounds:
-        raise NotImplementedError("Must be implemented by subclass")
+        msg = "Must be implemented by subclass"
+        raise NotImplementedError(msg)
 
 
 class Point(_Geometry):
@@ -190,14 +220,13 @@ class Point(_Geometry):
 
     A point has zero length and zero area.
 
-    Attributes
+    Attributes:
     ----------
     x, y, z : float
         Coordinate values
 
-    Example
+    Example:
     -------
-
       >>> p = Point(1.0, -1.0)
       >>> print p
       POINT (1.0 -1.0)
@@ -205,8 +234,11 @@ class Point(_Geometry):
       -1.0
       >>> p.x
       1.0
+
     """
 
+    _geoms: PointType
+
     def __init__(self, x: float, y: float, z: Optional[float] = None) -> None:
         """
         Initialize a Point.
@@ -215,72 +247,67 @@ class Point(_Geometry):
         ----------
         2 or 3 coordinate parameters: x, y, [z] : float
             Easting, northing, and elevation.
+
         """
-        self._coordinates = cast(
-            PointType,
-            tuple(
-                coordinate
-                for coordinate in (x, y, z)
-                if coordinate is not None and not math.isnan(coordinate)
-            ),
+        geoms = (x, y, z) if z is not None else (x, y)
+        object.__setattr__(
+            self,
+            "_geoms",
+            geoms,
         )
 
     def __repr__(self) -> str:
         """Return the representation."""
-        return f"{self.geom_type}{self._coordinates}"
+        if self.is_empty:
+            return f"{self.geom_type}()"
+        return f"{self.geom_type}{self._geoms}"
 
     @property
     def is_empty(self) -> bool:
         """
         Return if this geometry is empty.
 
-        A Point is considered empty when it has less than 2 coordinates.
+        A Point is considered empty when it has no valid coordinates.
         """
-        return len(self._coordinates) < 2
+        return any(coord is None or math.isnan(coord) for coord in self._geoms)
 
     @property
     def x(self) -> float:
         """Return x coordinate."""
-        return self._coordinates[0]
+        return self._geoms[0]
 
     @property
     def y(self) -> float:
         """Return y coordinate."""
-        return self._coordinates[1]
+        return self._geoms[1]
 
     @property
     def z(self) -> Optional[float]:
         """Return z coordinate."""
-        return (
-            self._coordinates[2]  # type: ignore[misc]
-            if len(self._coordinates) == 3
-            else None
-        )
+        if self.has_z:
+            return self._geoms[2]  # type: ignore [misc]
+        msg = f"The {self!r} geometry does not have z values"
+        raise DimensionError(msg)
 
     @property
-    def coords(self) -> Tuple[PointType]:
+    def coords(self) -> Union[Tuple[PointType], Tuple[()]]:
         """Return the geometry coordinates."""
-        return (self._coordinates,)
+        return () if self.is_empty else (self._geoms,)
 
     @property
     def has_z(self) -> bool:
         """Return True if the geometry's coordinate sequence(s) have z values."""
-        return len(self._coordinates) == 3
+        return len(self._geoms) == 3  # noqa: PLR2004
 
     @property
     def _wkt_coords(self) -> str:
-        return " ".join(str(coordinate) for coordinate in self._coordinates)
-
-    @property
-    def _wkt_inset(self) -> str:
-        """Return Z for 3 dimensional geometry or an empty string for 2 dimensions."""
-        return " Z " if len(self._coordinates) == 3 else " "
+        return " ".join(str(coordinate) for coordinate in self._geoms)
 
     @property
     def __geo_interface__(self) -> GeoInterface:
         """Return the geo interface."""
         geo_interface = super().__geo_interface__
-        geo_interface["coordinates"] = cast(PointType, tuple(self._coordinates))
+        geo_interface["coordinates"] = cast(PointType, tuple(self._geoms))
         return geo_interface
 
     @classmethod
@@ -311,8 +338,11 @@ class LineString(_Geometry):
     ----------
     geoms : sequence
         A sequence of Points
+
     """
 
+    _geoms: Tuple[Point, ...]
+
     def __init__(self, coordinates: LineType) -> None:
         """
         Initialize a Linestring.
@@ -327,8 +357,9 @@ class LineString(_Geometry):
         Create a line with two segments
 
           >>> a = LineString([(0, 0), (1, 0), (1, 1)])
+
         """
-        self._geoms = self._set_geoms(coordinates)
+        object.__setattr__(self, "_geoms", self._set_geoms(coordinates))
 
     def __repr__(self) -> str:
         """Return the representation."""
@@ -342,40 +373,28 @@ class LineString(_Geometry):
     @property
     def coords(self) -> LineType:
         """Return the geometry coordinates."""
-        return tuple(point.coords[0] for point in self.geoms)
+        return cast(
+            LineType,
+            tuple(point.coords[0] for point in self.geoms if point.coords),
+        )
 
     @property
     def is_empty(self) -> bool:
         """
         Return if this geometry is empty.
 
-        A Linestring is considered empty when it has less than 2 points.
+        A Linestring is considered empty when it has no points.
         """
-        return len(self._geoms) < 2
+        return len(self._geoms) == 0
 
     @property
     def has_z(self) -> Optional[bool]:
         """Return True if the geometry's coordinate sequence(s) have z values."""
         return self._geoms[0].has_z if self.geoms else None
 
-    @property
-    def maybe_valid(self) -> bool:
-        """
-        Check validity of the coordinates.
-
-        Returns False if the coordinates collapse to a single Point.
-        This only highlights obvious problems with this geometry.
-        Even if this test passes the geometry may still be invalid.
-        """
-        return len({p.coords[0] for p in self._geoms}) > 1
-
-    @property
-    def _wkt_inset(self) -> str:
-        return self.geoms[0]._wkt_inset
-
     @property
     def _wkt_coords(self) -> str:
-        return ", ".join(point._wkt_coords for point in self.geoms)
+        return ", ".join(point._wkt_coords for point in self.geoms)  # noqa: SLF001
 
     @property
     def __geo_interface__(self) -> GeoInterface:
@@ -385,14 +404,16 @@ class LineString(_Geometry):
         return geo_interface
 
     @classmethod
-    def from_coordinates(cls, coordinates: Sequence[PointType]) -> "LineString":
+    def from_coordinates(cls, coordinates: LineType) -> "LineString":
         """Construct a linestring from coordinates."""
         return cls(coordinates)
 
     @classmethod
     def from_points(cls, *args: Point) -> "LineString":
         """Create a linestring from points."""
-        return cls(tuple(point.coords[0] for point in args))
+        return cls(
+            cast(LineType, tuple(point.coords[0] for point in args if point.coords)),
+        )
 
     @classmethod
     def _from_dict(cls, geo_interface: GeoInterface) -> "LineString":
@@ -405,12 +426,15 @@ class LineString(_Geometry):
         last_len = None
         for coord in dedupe(coordinates):
             if len(coord) != last_len and last_len is not None:
+                msg = (  # type: ignore [unreachable]
+                    "All coordinates must have the same dimension"
+                )
                 raise DimensionError(
-                    "All coordinates must have the same dimension",
+                    msg,
                 )
             last_len = len(coord)
             point = Point(*coord)
-            if not point.is_empty:
+            if point:
                 geoms.append(point)
         return tuple(geoms)
 
@@ -443,25 +467,28 @@ class LinearRing(LineString):
         Initialize a LinearRing.
 
         Args:
+        ----
             coordinates (Sequence):
                 A sequence of (x, y [,z]) numeric coordinate pairs or triples
+
         """
         super().__init__(coordinates)
         if not self.is_empty and self._geoms[0].coords != self._geoms[-1].coords:
-            self._geoms = self._geoms + (self._geoms[0],)
+            object.__setattr__(self, "_geoms", (*self._geoms, self._geoms[0]))
 
     @property
     def centroid(self) -> Optional[Point]:
         """Return the centroid of the ring."""
         if self.has_z:
-            raise DimensionError("Centeroid is only implemented for 2D coordinates")
-        try:
-            cent, area = centroid(self.coords)
-        except ZeroDivisionError:
+            msg = "Centeroid is only implemented for 2D coordinates"
+            raise DimensionError(msg)
+
+        cent, area = centroid(self.coords)
+        if any(math.isnan(coord) for coord in cent):
             return None
         return (
-            Point(cent[0], cent[1])
-            if math.isclose(area, signed_area(self.coords))
+            Point(x=cent[0], y=cent[1])
+            if math.isclose(a=area, b=signed_area(self.coords))
             else None
         )
 
@@ -470,25 +497,6 @@ class LinearRing(LineString):
         """Return True if the ring is oriented counter clock-wise."""
         return signed_area(self.coords) >= 0
 
-    @property
-    def maybe_valid(self) -> bool:
-        """
-        Check validity of the coordinates.
-
-        This only highlights obvious problems with this geometry.
-        Even if this test passes the geometry may still be invalid.
-        """
-        if self.has_z:
-            raise DimensionError("Validation is only implemented for 2D coordinates")
-        min_x, min_y, max_x, max_y = self.bounds  # type: ignore [misc]
-        if min_x == max_x or min_y == max_y:
-            return False
-        try:
-            _, area = centroid(self.coords)
-        except ZeroDivisionError:
-            return False
-        return math.isclose(area, signed_area(self.coords))
-
 
 class Polygon(_Geometry):
     """
@@ -504,8 +512,11 @@ class Polygon(_Geometry):
         The ring which bounds the positive space of the polygon.
     interiors : sequence
         A sequence of rings which bound all existing holes.
+
     """
 
+    _geoms: Tuple[LinearRing, ...]
+
     def __init__(
         self,
         shell: LineType,
@@ -528,11 +539,11 @@ class Polygon(_Geometry):
 
           >>> coords = ((0., 0.), (0., 1.), (1., 1.), (1., 0.), (0., 0.))
           >>> polygon = Polygon(coords)
+
         """
-        self._interiors: Tuple[LinearRing, ...] = ()
-        if holes:
-            self._interiors = tuple(LinearRing(hole) for hole in holes)
-        self._exterior = LinearRing(shell)
+        interiors = tuple(LinearRing(hole) for hole in holes) if holes else ()
+        exterior = LinearRing(shell)
+        object.__setattr__(self, "_geoms", (exterior, interiors))
 
     def __repr__(self) -> str:
         """Return the representation."""
@@ -541,12 +552,16 @@ class Polygon(_Geometry):
     @property
     def exterior(self) -> LinearRing:
         """Return the exterior Linear Ring of the polygon."""
-        return self._exterior
+        return self._geoms[0]
 
     @property
     def interiors(self) -> Iterator[LinearRing]:
         """Interiors (Holes) of the polygon."""
-        yield from (interior for interior in self._interiors if interior)
+        yield from (
+            interior
+            for interior in self._geoms[1]  # type: ignore [attr-defined]
+            if interior
+        )
 
     @property
     def is_empty(self) -> bool:
@@ -555,7 +570,7 @@ class Polygon(_Geometry):
 
         A polygon is empty when it does not have an exterior.
         """
-        return self._exterior.is_empty
+        return self._geoms[0].is_empty
 
     @property
     def coords(self) -> PolygonType:
@@ -564,48 +579,34 @@ class Polygon(_Geometry):
 
         Note that this is not implemented in Shapely.
         """
-        if self._interiors:
-            return self.exterior.coords, tuple(
-                interior.coords for interior in self.interiors if interior
+        if self._geoms[1]:
+            return cast(
+                PolygonType,
+                (
+                    self.exterior.coords,
+                    tuple(interior.coords for interior in self.interiors if interior),
+                ),
             )
-        return (self.exterior.coords,)
+        return cast(PolygonType, (self.exterior.coords,))
 
     @property
     def has_z(self) -> Optional[bool]:
         """Return True if the geometry's coordinate sequence(s) have z values."""
-        return self._exterior.has_z
-
-    @property
-    def maybe_valid(self) -> bool:
-        """
-        Check validity of the coordinates.
-
-        This only highlights obvious problems with this geometry.
-        Even if this test passes the geometry may still be invalid.
-        """
-        if not self._check_interior_bounds():
-            return False
-        return (
-            all(interior.maybe_valid for interior in self.interiors)
-            if self._exterior.maybe_valid
-            else False
-        )
+        return self._geoms[0].has_z
 
     @property
     def _wkt_coords(self) -> str:
-        ec = self.exterior._wkt_coords
-        ic = "".join(f",({interior._wkt_coords})" for interior in self.interiors)
+        ec = self.exterior._wkt_coords  # noqa: SLF001
+        ic = "".join(
+            f",({interior._wkt_coords})" for interior in self.interiors  # noqa: SLF001
+        )
         return f"({ec}){ic}"
 
-    @property
-    def _wkt_inset(self) -> str:
-        return self.exterior._wkt_inset
-
     @property
     def __geo_interface__(self) -> GeoInterface:
         """Return the geo interface."""
         geo_interface = super().__geo_interface__
-        coords = (self.exterior.coords,) + tuple(hole.coords for hole in self.interiors)
+        coords = (self.exterior.coords, *tuple(hole.coords for hole in self.interiors))
         geo_interface["coordinates"] = coords
         return geo_interface
 
@@ -617,37 +618,26 @@ class Polygon(_Geometry):
     @classmethod
     def from_linear_rings(cls, shell: LinearRing, *args: LinearRing) -> "Polygon":
         """Construct a Polygon from linear rings."""
-        return cls(shell.coords, tuple(lr.coords for lr in args))
+        return cls(
+            shell=shell.coords,
+            holes=tuple(lr.coords for lr in args),
+        )
 
     @classmethod
     def _from_dict(cls, geo_interface: GeoInterface) -> "Polygon":
         cls._check_dict(geo_interface)
+        if not geo_interface["coordinates"]:
+            return cls(shell=(), holes=())
         return cls(
-            cast(LineType, geo_interface["coordinates"][0]),
-            cast(Tuple[LineType], geo_interface["coordinates"][1:]),
+            shell=cast(LineType, geo_interface["coordinates"][0]),
+            holes=cast(Tuple[LineType], geo_interface["coordinates"][1:]),
         )
 
-    def _check_interior_bounds(self) -> bool:
-        """Check that the bounding boxes of holes are inside the bounds of the shell."""
-        bounds = self.bounds
-        if not bounds:
-            return False
-        for interior in self.interiors:
-            i_box = cast(Bounds, interior.bounds)
-            if (
-                bounds[0] > i_box[0]
-                or bounds[1] > i_box[1]
-                or bounds[2] < i_box[2]
-                or bounds[3] < i_box[3]
-            ):
-                return False
-        return True
-
     def _get_bounds(self) -> Bounds:
-        return self.exterior._get_bounds()
+        return self.exterior._get_bounds()  # noqa: SLF001
 
     def _prepare_hull(self) -> Iterable[Point2D]:
-        return self.exterior._prepare_hull()
+        return self.exterior._prepare_hull()  # noqa: SLF001
 
 
 class _MultiGeometry(_Geometry):
@@ -664,18 +654,15 @@ class _MultiGeometry(_Geometry):
 
         Multi-part geometries do not provide a coordinate sequence.
         """
+        msg = "Multi-part geometries do not provide a coordinate sequence"
         raise NotImplementedError(
-            "Multi-part geometries do not provide a coordinate sequence",
+            msg,
         )
 
     @property
     def has_z(self) -> Optional[bool]:
         """Return True if any geometry of the collection have z values."""
-        return (
-            any(geom.has_z for geom in self.geoms)
-            if self._geoms  # type: ignore [attr-defined]
-            else None
-        )
+        return any(geom.has_z for geom in self.geoms) if self._geoms else None
 
     @property
     def geoms(self) -> Iterator[_Geometry]:
@@ -712,8 +699,11 @@ class MultiPoint(_MultiGeometry):
     ----------
     geoms : sequence
         A sequence of Points
+
     """
 
+    _geoms: Tuple[Point, ...]
+
     def __init__(self, points: Sequence[PointType], unique: bool = False) -> None:
         """
         Create a collection of one or more points.
@@ -735,10 +725,11 @@ class MultiPoint(_MultiGeometry):
           2
           >>> type(ob.geoms[0]) == Point
           True
+
         """
         if unique:
             points = set(points)  # type: ignore [assignment]
-        self._geoms = tuple(Point(*point) for point in points)
+        object.__setattr__(self, "_geoms", tuple(Point(*point) for point in points))
 
     def __len__(self) -> int:
         """Return the number of points in this MultiPoint."""
@@ -746,7 +737,10 @@ class MultiPoint(_MultiGeometry):
 
     def __repr__(self) -> str:
         """Return the representation."""
-        return f"{self.geom_type}({tuple(geom.coords[0] for geom in self._geoms)})"
+        return (
+            f"{self.geom_type}"
+            f"({tuple(geom.coords[0] for geom in self._geoms if geom.coords)})"
+        )
 
     @property
     def geoms(self) -> Iterator[Point]:
@@ -755,19 +749,21 @@ class MultiPoint(_MultiGeometry):
 
     @property
     def _wkt_coords(self) -> str:
-        return ", ".join(point._wkt_coords for point in self.geoms)
+        return ", ".join(point._wkt_coords for point in self.geoms)  # noqa: SLF001
 
     @property
     def __geo_interface__(self) -> GeoInterface:
         """Return the geo interface."""
         geo_interface = super().__geo_interface__
-        geo_interface["coordinates"] = tuple(geom.coords[0] for geom in self.geoms)
+        geo_interface["coordinates"] = tuple(
+            geom.coords[0] for geom in self.geoms if geom.coords
+        )
         return geo_interface
 
     @classmethod
     def from_points(cls, *args: Point, unique: bool = False) -> "MultiPoint":
         """Create a MultiPoint from Points."""
-        return cls([point.coords[0] for point in args], unique=unique)
+        return cls([point.coords[0] for point in args if point.coords], unique=unique)
 
     @classmethod
     def _from_dict(cls, geo_interface: GeoInterface) -> "MultiPoint":
@@ -788,8 +784,11 @@ class MultiLineString(_MultiGeometry):
     ----------
     geoms : sequence
         A sequence of LineStrings
+
     """
 
+    _geoms: Tuple[LineString, ...]
+
     def __init__(self, lines: Sequence[LineType], unique: bool = False) -> None:
         """
         Initialize the MultiLineString.
@@ -807,10 +806,11 @@ class MultiLineString(_MultiGeometry):
         Construct a collection containing one line string.
 
           >>> lines = MultiLineString( [[[0.0, 0.0], [1.0, 2.0]]] )
+
         """
         if unique:
             lines = {tuple(line) for line in lines}  # type: ignore [assignment]
-        self._geoms = tuple(LineString(line) for line in lines)
+        object.__setattr__(self, "_geoms", tuple(LineString(line) for line in lines))
 
     def __len__(self) -> int:
         """Return the number of lines in the collection."""
@@ -827,7 +827,9 @@ class MultiLineString(_MultiGeometry):
 
     @property
     def _wkt_coords(self) -> str:
-        return ",".join(f"({linestring._wkt_coords})" for linestring in self.geoms)
+        return ",".join(
+            f"({linestring._wkt_coords})" for linestring in self.geoms  # noqa: SLF001
+        )
 
     @property
     def __geo_interface__(self) -> GeoInterface:
@@ -868,8 +870,11 @@ class MultiPolygon(_MultiGeometry):
     ----------
     geoms : sequence
         A sequence of `Polygon` instances
+
     """
 
+    _geoms: Tuple[Polygon, ...]
+
     def __init__(self, polygons: Sequence[PolygonType], unique: bool = False) -> None:
         """
         Initialize a Multipolygon.
@@ -899,16 +904,21 @@ class MultiPolygon(_MultiGeometry):
           1
           >>> type(ob.geoms[0]) == Polygon
           True
+
         """
         if unique:
             polygons = set(polygons)  # type: ignore [assignment]
 
-        self._geoms = tuple(
-            Polygon(
-                polygon[0],
-                polygon[1] if len(polygon) == 2 else None,  # type: ignore [misc]
-            )
-            for polygon in polygons
+        object.__setattr__(
+            self,
+            "_geoms",
+            tuple(
+                Polygon(
+                    shell=polygon[0],
+                    holes=polygon[1] if len(polygon) == 2 else None,  # noqa: PLR2004
+                )
+                for polygon in polygons
+            ),
         )
 
     def __len__(self) -> int:
@@ -926,14 +936,14 @@ class MultiPolygon(_MultiGeometry):
 
     @property
     def _wkt_coords(self) -> str:
-        return ",".join(f"({poly._wkt_coords})" for poly in self.geoms)
+        return ",".join(f"({poly._wkt_coords})" for poly in self.geoms)  # noqa: SLF001
 
     @property
     def __geo_interface__(self) -> GeoInterface:
         """Return the geo interface."""
         geo_interface = super().__geo_interface__
         coords = tuple(
-            (geom.exterior.coords,) + tuple(hole.coords for hole in geom.interiors)
+            (geom.exterior.coords, *tuple(hole.coords for hole in geom.interiors))
             for geom in self.geoms
         )
         geo_interface["coordinates"] = coords
@@ -975,7 +985,7 @@ class GeometryCollection(_MultiGeometry):
     """
     A heterogenous collection of geometries.
 
-    Attributes
+    Attributes:
     ----------
     geoms : sequence
         A sequence of geometry instances
@@ -985,9 +995,8 @@ class GeometryCollection(_MultiGeometry):
     class isn't generally supported by ordinary GIS sw (viewers and so on). So
     it's very rarely used in the real GIS professional world.
 
-    Example
+    Example:
     -------
-
     Initialize Geometries and construct a GeometryCollection
 
     >>> from pygeoif import geometry
@@ -999,16 +1008,24 @@ class GeometryCollection(_MultiGeometry):
     {'type': 'GeometryCollection',
     'geometries': [{'type': 'Point', 'coordinates': (1.0, -1.0)},
     {'type': 'Point', 'coordinates': (1.0, -1.0)}]}
+
     """
 
-    def __init__(self, geometries: Iterable[Geometry]) -> None:
+    _geoms: Tuple[Union[Geometry, "GeometryCollection"], ...]
+
+    def __init__(
+        self,
+        geometries: Iterable[Union[Geometry, "GeometryCollection"]],
+    ) -> None:
         """
         Initialize the MultiGeometry with Geometries.
 
         Args:
+        ----
             geometries (Iterable[Geometry]
+
         """
-        self._geoms = tuple(geom for geom in geometries if geom)
+        object.__setattr__(self, "_geoms", tuple(geom for geom in geometries if geom))
 
     def __eq__(self, other: object) -> bool:
         """
@@ -1017,6 +1034,8 @@ class GeometryCollection(_MultiGeometry):
         Types and coordinates from all contained geometries must be equal.
         """
         try:
+            if self.is_empty:
+                return False
             if (
                 other.__geo_interface__.get("type")  # type: ignore [attr-defined]
                 != self.geom_type
@@ -1033,26 +1052,19 @@ class GeometryCollection(_MultiGeometry):
                 return False
         except AttributeError:
             return False
-        return all(
-            (
-                s["type"] == o.get("type")
-                and compare_coordinates(s["coordinates"], o.get("coordinates"))
-                for s, o in zip(
-                    (geom.__geo_interface__ for geom in self.geoms),
-                    other.__geo_interface__.get(  # type:  ignore [attr-defined]
-                        "geometries",
-                        [],
-                    ),
-                )
-            ),
+        return compare_geo_interface(
+            first=self.__geo_interface__,
+            second=other.__geo_interface__,  # type: ignore [attr-defined]
         )
 
     def __len__(self) -> int:
         """
         Length of the collection.
 
-        Returns:
+        Returns
+        -------
             int: Number of geometries in the collection.
+
         """
         return len(self._geoms)
 
@@ -1068,12 +1080,14 @@ class GeometryCollection(_MultiGeometry):
     def __geo_interface__(self) -> GeoCollectionInterface:  # type: ignore [override]
         """Return the geo interface of the collection."""
         return {
-            "type": self.geom_type,
+            "type": "GeometryCollection",
             "geometries": tuple(geom.__geo_interface__ for geom in self.geoms),
         }
 
     def _prepare_hull(self) -> Iterable[Point2D]:
-        return chain.from_iterable(geom._prepare_hull() for geom in self.geoms)
+        return chain.from_iterable(
+            geom._prepare_hull() for geom in self.geoms  # noqa: SLF001
+        )
 
 
 __all__ = [
